#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import mock
import json
import imp
import unittest
import urllib2

from tests import capture_stdout


class GetTicketStatus(unittest.TestCase):

    def setUp(self):
        super(GetTicketStatus, self).setUp()
        self.cli = imp.load_source(
            "ubuntu-ci",
            os.path.join(os.path.dirname(__file__), "../ubuntu-ci")
        )

    @mock.patch('urllib2.urlopen')
    def test_get_single_ticket_status(self, mock_urlopen):
        data = {
            "current_workflow_step": 100,
            "status": 110,
            "title": "My cool feature",
        }
        resp = mock.Mock()
        resp.read.return_value = json.dumps(data)
        mock_urlopen.return_value = resp
        args = self.cli.parse_arguments(['status', '-t', '100'])
        with capture_stdout(args.func, args) as cm:
            self.assertEqual("Ticket #100, My cool feature is 100 110\n",
                             cm)
        mock_urlopen.assert_called_once()

    @mock.patch('urllib2.urlopen')
    def test_get_ticket_status_unicode(self, mock_urlopen):
        data = {
            "current_workflow_step": 100,
            "status": 110,
            "title": "Testing unicode in title: joão",
        }
        resp = mock.Mock()
        resp.read.return_value = json.dumps(data)
        mock_urlopen.return_value = resp
        args = self.cli.parse_arguments(['status', '-t', '100'])
        with capture_stdout(args.func, args) as cm:
            self.assertEqual("Ticket #100, Testing unicode in title: joão is "
                             "100 110\n", cm)
        mock_urlopen.assert_called_once()

    @mock.patch('urllib2.urlopen')
    def test_get_all_ticket_status(self, mock_urlopen):
        data = {
            "objects": [
                {
                    "id": 5,
                    "current_workflow_step": 100,
                    "status": 110,
                    "title": "My cool feature",
                },
                {
                    "id": 6,
                    "current_workflow_step": 100,
                    "status": 110,
                    "title": "Ubuntu bug fix",
                },
                {
                    "id": 7,
                    "current_workflow_step": 100,
                    "status": 110,
                    "title": "Upstream bug fix",
                },
                {
                    "id": 9,
                    "current_workflow_step": 100,
                    "status": 110,
                    "title": "Other new feature",
                },
            ]
        }
        resp = mock.Mock()
        resp.read.return_value = json.dumps(data)
        mock_urlopen.return_value = resp
        args = self.cli.parse_arguments(['status'])
        with capture_stdout(args.func, args) as cm:
            self.assertEqual(
                "Ticket #5, My cool feature is 100 110\n" +
                "Ticket #6, Ubuntu bug fix is 100 110\n" +
                "Ticket #7, Upstream bug fix is 100 110\n" +
                "Ticket #9, Other new feature is 100 110\n",
                cm)
        mock_urlopen.assert_called_once()

    @mock.patch('urllib2.urlopen')
    def test_get_status_404_response(self, mock_urlopen):
        mock_urlopen.side_effect = urllib2.HTTPError("http://example.com",
                                                     404, "Not Found", "",
                                                     None)
        args = self.cli.parse_arguments(['status', '-t', '99'])
        with self.assertRaises(SystemExit) as cm:
            args.func(args)
        mock_urlopen.assert_called_once()
        self.assertEqual("Ticket number 99 not found.", cm.exception.message)
