"use strict";

YUI.add('webui', function (Y) {
    var json_error_msg = "The ticket system returned data that could not be understood. " +
                         "Please try refreshing, or contact support if this problem persists.",
        refresh_msg = "Refreshing data...";
    Y.webui = {
        url_prefix: "/api/v1/",
        refresh_interval: (parseInt(Y.one('#refresh_rate'), 10) || 60) * 1000,
        done_loading: false,
        add_refresh_div: function (callback) {
            var refresh,
                refresh_rate,
                refresh_enabled,
                onChange;

            refresh = Y.Node.create('<div id="refresh"></div>');

            refresh.setHTML('<small>Auto refresh <input type="checkbox" checked="true"' +
                'id="refresh_enabled"></input> every <input type="text"' +
                'id="refresh_rate" value="60" maxlength="3" size="2"/> seconds.</small>'
                );

            Y.one('div#main-content').append(refresh);

            Y.webui.refresh_timeout = null;
            refresh_rate = Y.one("#refresh_rate");
            refresh_enabled = Y.one("#refresh_enabled");
            Y.webui.refresh_enabled = refresh_enabled.get('checked');

            onChange = function () {
                var value = refresh_rate.get('value'),
                    val;

                val = parseInt(value, 10);

                if (Y.webui.refresh_timeout) {
                    clearTimeout(Y.webui.refresh_timeout);
                }
                if (val !== 0) {
                    Y.webui.refresh_interval = val * 1000;
                    callback();
                }
            };
            refresh_rate.on('change', onChange);
            refresh_enabled.on('change', function () {
                if (Y.webui.refresh_timeout) {
                    clearTimeout(Y.webui.refresh_timeout);
                }

                Y.webui.refresh_enabled = refresh_enabled.get('checked');

                if (Y.webui.refresh_enabled) {
                    callback();
                }
            });
            onChange();
        },
        get_workflow_data: function (api_url) {
            var wf_data = {},
                wf_str = {},
                wf_num = {},
                i,
                item;

            Y.io(Y.webui.url_prefix + api_url, {
                sync: true,
                on: {
                    success: function (tx, r) {
                        try {
                            wf_data = Y.JSON.parse(r.responseText).objects;
                        } catch (e) {
                            Y.webui.main_table.showMessage(json_error_msg);
                            return;
                        }
                    }
                }
            });

            for (i = 0; i < wf_data.length; i += 1) {
                item = wf_data[i];

                wf_str[item.name] = item.value;
                wf_num[item.value] = item.name;
            }

            return {
                wf_data_list: wf_data,
                wf_data_list_str: wf_str,
                wf_data_list_num: wf_num
            };
        },
        get_workflow_statuses: function () {
            var result, api_url = "ticket_workflow_step_status/";

            result = Y.webui.get_workflow_data(api_url);

            Y.webui.workflow_status_list = result.wf_data_list;
            Y.webui.workflow_status_list_str = result.wf_data_list_str;
            Y.webui.workflow_status_list_num = result.wf_data_list_num;
        },
        get_workflow_steps: function () {
            var result, api_url = "ticket_workflow_step/";

            result = Y.webui.get_workflow_data(api_url);

            Y.webui.workflow_step_list = result.wf_data_list;
            Y.webui.workflow_step_list_str = result.wf_data_list_str;
            Y.webui.workflow_step_list_num = result.wf_data_list_num;
        },
        get_workflow_step_from_status: function (wf_status_str) {
            var status_num, step_num, step_name, step_factor;

            status_num = Y.webui.workflow_status_list_str[wf_status_str];

            step_factor = Math.floor(status_num / 100);
            step_num = step_factor * 100;

            step_name = Y.webui.workflow_step_list_num[step_num];

            return step_name;
        },
        get_ticket: function (ticket_id, elem) {

            if (Y.webui.main_table) {
                Y.webui.main_table.showMessage("loading data...");
            }
            Y.io(Y.webui.url_prefix + "fullticket/" + ticket_id + "/", {
                on: {
                    success: function (tx, r) {
                        console.log("success");
                        var data, new_div;

                        try {
                            data = Y.JSON.parse(r.responseText);
                        } catch (e) {
                            Y.webui.main_table.showMessage(json_error_msg);
                            return;
                        }

                        new_div = Y.webui.ticket_detail_element(data);
                        elem.setHTML(new_div);

                    },
                    failure: function (tx, r, e) {
                        console.log("failure: ", tx, r, e);
                    }
                }
            });
        },
        get_tickets: function () {

            if (Y.webui.main_table) {
                Y.webui.main_table.showMessage("loading data...");
            }
            Y.io(Y.webui.url_prefix + "opentickets/", {
                on: {
                    success: function (tx, r) {
                        var data,
                            i;
                        console.log("success");

                        try {
                            data = Y.JSON.parse(r.responseText);
                        } catch (e) {
                            Y.webui.main_table.showMessage(json_error_msg);
                            return;
                        }

                        if (data.objects && data.objects.length !== 0) {
                            for (i = 0; i < data.objects.length; i += 1) {
                                if (data.objects[i]) {
                                    Y.webui.add_item(data.objects[i]);
                                }
                            }
                            Y.webui.check_statuses();
                        }
                        if (Y.webui.main_table.data.size() === 0) {
                            Y.webui.main_table.showMessage("No results found");
                        }
                        Y.webui.done_loading = true;
                    },
                    failure: function (tx, r, e) {
                        console.log("failure: ", tx, r, e);
                    }
                }
            });
        },
        add_subticket: function (data) {
            var main_tr = Y.Node.create("<tr>"),
                key,
                value,
                tr;

            main_tr.appendChild(Y.Node.create('<th>subticket</th>'));
            for (key in data) {
                if (data.hasOwnProperty(key)) {
                    value = data[key];
                    tr = main_tr.appendChild(Y.Node.create('<tr>'));
                    tr.addClass('subticket');
                    tr.appendChild(Y.Node.create('<th>' + key + '</th>'));
                    tr.appendChild(Y.Node.create('<td>' + value + '</td>'));
                }
            }

            main_tr.addClass('subticket');
            return main_tr;
        },
        make_subticket: function (data) {
            var div = Y.Node.create("<div class='subticket'></div>"),
                table = div.appendChild(Y.Node.create("<table>")),
                hidden_fields = Y.Array(['id', 'resource_uri', 'current_workflow_step', 'status', 'source_package_upload']),
                key,
                value,
                tr,
                span;

            for (key in data) {
                if (data.hasOwnProperty(key)) {
                    value = data[key];

                    // Skip hidden fields
                    if (Y.Array.indexOf(hidden_fields, key) !== -1) {
                        continue;
                    }

                    if (key === 'artifact') {
                        span = Y.webui.add_artifacts(data.artifact);
                        value = span.getHTML();
                    }

                    if (key === 'source_package_upload') {
                        span = Y.webui.sp_upload(data.source_package_upload);
                        value = span.getHTML();
                    }

                    tr = table.appendChild(Y.Node.create('<tr>'));
                    tr.appendChild(Y.Node.create('<th>' + key + ': </th>'));
                    tr.appendChild(Y.Node.create('<td>' + value + '</td>'));
                }
            }

            return div;
        },
        basename: function (path) {
            return path.slice(path.lastIndexOf('/') + 1, path.length);
        },
        add_artifacts: function (data) {
            var span = Y.Node.create('<span></span>'),
                i;

            for (i = 0; i < data.length; i += 1) {
                span.appendChild(Y.webui.artifact_link(data[i]));
                span.appendChild(Y.Node.create("<br/>"));
            }

            return span;
        },
        artifact_link: function (data) {
            var name = Y.webui.basename(data.name),
                link = Y.Node.create("<a/>");

            link.set('href', data.reference);
            link.set('text', name);

            return link;
        },
        sp_upload: function (data) {
            var span = Y.Node.create("<span></span>");

            if (data.sourcepackage) {
                span.setHTML(data.sourcepackage.name + " " + data.version);
            }

            return span;
        },
        ticket_detail_element: function (data) {
            var hidden_fields = Y.Array(
                    [
                        'id', 'title', 'subticket', 'resource_uri',
                        'current_workflow_step', 'status', 'owner', 'description'
                    ]
                ),
                current_step,
                div,
                fail_str,
                found_step,
                i,
                inner_div,
                key,
                not_yet_step,
                short_status,
                skipped_step,
                span,
                status_image,
                step_status,
                subt,
                subticket,
                subticket_count,
                subticket_wrapper,
                table,
                td,
                td1,
                ticket_div,
                ticket_row_div,
                tr,
                value,
                wf_data,
                wf_table;

            div = Y.Node.create("<div id='table_wrapper'/>");
            ticket_row_div = div.appendChild(Y.Node.create("<div class='ticket_row'></div>"));
            ticket_row_div.append(Y.Node.create('<h2>#'    + data.id + ' - ' + data.title + '</h2>'));
            ticket_div = ticket_row_div.appendChild(Y.Node.create("<div class='ticket'></div>"));
            table = ticket_div.appendChild(Y.Node.create("<table></table>"));

            if (data.title) {
                inner_div = Y.Node.create("<div class='workflow'></div>");
                wf_table = Y.Node.create('<table class="workflow_steps"></table>');
                fail_str = "Failed";

                wf_table.appendChild(Y.Node.create('<tr><th>Landing Progress</th>' +
                    '<th class="workflow_status">Status</th>' +
                    '</tr></table>')
                    );
                ticket_row_div.append(inner_div);

                wf_data = Y.webui.workflow_step_list;

                short_status = data.status.slice(data.status.length - fail_str.length, data.status.length);

                found_step = false;
                if (data.current_workflow_step !== "Failed") {
                    current_step = data.current_workflow_step;
                } else {
                    current_step = Y.webui.get_workflow_step_from_status(data.status);
                }
                for (i = 0; i < wf_data.length; i += 1) {
                    step_status = 'Completed';
                    status_image = null;
                    skipped_step = false;

                    if (!found_step) {
                        step_status = 'Completed';
                        status_image = 'check.svg';
                    }

                    if (wf_data[i].name === "Completed" || wf_data[i].name === "Failed" ||
                            wf_data[i].name === "New") {
                        skipped_step = true;
                    }

                    if (!found_step && (current_step === wf_data[i].name)) {
                        found_step = true;
                        if (data.status !== "Completed" && short_status !== "Failed") {
                            step_status = "In Progress";
                            status_image = 'spinner3.gif';
                        } else {
                            step_status = data.status;
                            if (short_status === "Failed") {
                                status_image = "cross.svg";
                            } else {
                                status_image = "check.svg";
                            }
                        }
                    }

                    if (not_yet_step) {
                        step_status = "Failed";
                        //status_image = "cross.svg";
                    }

                    if (data.current_workflow_step === "Failed" && wf_data[i].name === "Failed") {
                        step_status = "Failed";
                        status_image = "cross.svg";
                    }

                    if (!skipped_step) {
                        tr = Y.Node.create('<tr></tr>');
                        wf_table.appendChild(tr);
                        td1 = Y.Node.create('<td class="workflow_col1">' + wf_data[i].name + ': </td>');
                        if (found_step && !not_yet_step) {
                            td1.addClass('current_step');
                        }
                        tr.appendChild(td1);
                        td = tr.appendChild('<td></td>');
                        if (status_image) {
                            td.appendChild('<img src="' + status_image +
                                    '" alt="' + step_status +
                                    '" title="' + step_status + '">');
                        }
                    }

                    if (found_step) {
                        not_yet_step = true;
                    }
                }

                if (short_status === "Failed") {
                    wf_table.appendChild(Y.Node.create("<tr></tr>")).appendChild(
                        Y.Node.create("<td colspan='2' class='red'>" + short_status + "</td>")
                    );
                }

                if (current_step === "Completed") {
                    wf_table.appendChild(Y.Node.create("<tr></tr>")).appendChild(
                        Y.Node.create("<td colspan='2' class='green'>Success</td>")
                    );
                }

                inner_div.append(wf_table);
            }

            for (key in data) {
                if (data.hasOwnProperty(key)) {
                    value = data[key];

                    if (Y.Array.indexOf(hidden_fields, key) !== -1) {
                        continue;
                    }

                    if (key === "title" || key === "subticket") {
                        continue;
                    }

                    if (key === 'artifact') {
                        span = Y.webui.add_artifacts(data.artifact);
                        value = span.getHTML();
                    }
                    key = key.replace("_", " ");
                    if (value === null) {
                        value = "";
                    }
                    tr = Y.Node.create('<tr></tr>');
                    table.appendChild(tr);
                    tr.appendChild(Y.Node.create('<th>' + key.charAt(0).toUpperCase() + key.slice(1) + ': </th>'));
                    tr.appendChild(Y.Node.create('<td>' + value + '</td>'));
                }
            }

            subticket_wrapper = div.appendChild("<div class='subticket_wrapper'></div>");

            if (data.subticket) {
                subticket_count = data.subticket.length;

                subticket_wrapper.appendChild("<h3>Uploads</h3>");
                if (subticket_count > 0) {
                    for (i = 0; i < subticket_count; i += 1) {
                        subticket = data.subticket[i];

                        subticket_wrapper.appendChild("<h4>" + data.subticket[i].source_package_upload.sourcepackage.name + " " + data.subticket[i].source_package_upload.version + "</h4><div id='status'><strong>Landing Progress:</strong> " + data.subticket[i].current_workflow_step + " <strong>Status:</strong> " + data.subticket[i].status + "</div>");

                        subt = Y.webui.make_subticket(subticket);

                        subticket_wrapper.appendChild(subt);
                    }
                }
            }
            return div;
        },
        get_ticket_id: function () {
            var query = window.location.search.split('?')[1],
                ticket_id = null,
                ob;

            if (query) {
                ob = Y.QueryString.parse(query);
                ticket_id = ob.ticket_id;
            }

            return ticket_id;
        },
        update_ticket: function () {
            var ticket_id = Y.webui.get_ticket_id();

            if (ticket_id) {
                Y.webui.get_ticket(ticket_id, Y.one('#container'));
            }

            // we're done so allow more refreshes if needed.
            if (Y.webui.refresh_timeout) {
                clearTimeout(Y.webui.refresh_timeout);
                Y.webui.refresh_timeout = null;
            }

            if (Y.webui.refresh_enabled && !Y.webui.refresh_timeout && Y.webui.refresh_interval > 0) {
                Y.webui.refresh_timeout = setTimeout(Y.webui.update_ticket, Y.webui.refresh_interval);
            }
        },
        has_ticket: function (ticket_ids, id) {
            return ticket_ids[id] === true;
        },
        check_statuses: function () {
            var ticket_id_map = {},
                table_data = Y.webui.main_table.get('data').toJSON(),
                i,
                record,
                ticket_id,
                ticket_ids,
                ids;

            for (i = 0; i < table_data.length; i += 1) {
                record = table_data[i];

                ticket_id = record.ticket_id;

                if (record.status !== 'Completed' && record.current_workflow_step !== 'Completed') {
                    if (ticket_id) {
                        ticket_id_map[ticket_id] = true;
                    }
                }
            }

            // Check for new tickets and add them to the table.
            if (Y.webui.main_table && Y.webui.done_loading) {
                Y.io(Y.webui.url_prefix + "opentickets/", {
                    on: {
                        success: function (tx, r) {
                            var data,
                                j,
                                id;

                            try {
                                data = Y.JSON.parse(r.responseText).objects;
                            } catch (e) {
                                Y.webui.main_table.showMessage(json_error_msg);
                                return;
                            }

                            if (data && data.length !== 0) {
                                for (j = 0; j < data.length; j += 1) {
                                    id = data[j].id;
                                    if (!Y.webui.has_ticket(ticket_id_map, id)) {
                                        Y.webui.add_item(data[j]);
                                        ticket_id_map[id] = true;
                                    }
                                }
                            }
                        }
                    }
                });
            }

            ticket_ids = Object.keys(ticket_id_map);
            if (ticket_ids.length > 0) {
                ids = ticket_ids.join(';');
                Y.webui.main_table.showMessage(refresh_msg);
                Y.io(Y.webui.url_prefix +    "ticketstatus/set/" + ids + "/", {
                    on: {
                        success: function (tx, r) {
                            var data,
                                data_map = {},
                                recordset = Y.webui.main_table.data.toJSON(),
                                k,
                                id;

                            try {
                                data = Y.JSON.parse(r.responseText).objects;
                            } catch (e) {
                                Y.webui.main_table.showMessage(json_error_msg);
                                return;
                            }

                            for (k = 0; k < data.length; k += 1) {
                                data_map[data[k].id] = data[k];
                            }

                            for (k = 0; k < recordset.length; k += 1) {
                                id = recordset[k].ticket_id;

                                if (data_map[id]) {
                                    recordset[k] = data_map[id];
                                    recordset[k].ticket_id = recordset[k].id;
                                }
                            }

                            Y.webui.main_table.set('data', recordset);
                            Y.webui.main_table.render();

                            // we're done so allow more refreshes if needed.
                            if (Y.webui.refresh_timeout) {
                                clearTimeout(Y.webui.refresh_timeout);
                                Y.webui.refresh_timeout = null;
                            }
                            if (Y.webui.refresh_enabled && !Y.webui.refresh_timeout && Y.webui.refresh_interval > 0) {
                                Y.webui.refresh_timeout = setTimeout(Y.webui.check_statuses, Y.webui.refresh_interval);
                            }
                        },
                        complete: function () {
                            if (Y.webui.refresh_enabled && !Y.webui.refresh_timeout && Y.webui.refresh_interval > 0) {
                                Y.webui.refresh_timeout = setTimeout(Y.webui.check_statuses, Y.webui.refresh_interval);
                            }
                        }
                    }
                });
            } else {
                if (Y.webui.refresh_timeout) {
                    clearTimeout(Y.webui.refresh_timeout);
                    Y.webui.refresh_timeout = null;
                }
                if (Y.webui.refresh_enabled && !Y.webui.refresh_timeout && Y.webui.refresh_interval > 0) {
                    Y.webui.refresh_timeout = setTimeout(Y.webui.check_statuses, Y.webui.refresh_interval);
                }
            }
        },
        add_item: function (item) {
            var entry = item,
                table = Y.webui.main_table,
                id;

            id = entry.id;

            if (table) {
                table.addRow({
                    title: entry.title,
                    current_workflow_step: entry.current_workflow_step,
                    status: entry.status,
                    ticket_id: id
                });
            }
        },
        make_main_table: function (div_id) {

            var format_id = function (o) {
                    o.rowClass = "data-row-" + o.data.ticket_id;

                    return o.value;
                },
                columns = [
                    {key: 'title', label: 'Title', formatter: format_id},
                    {key: 'current_workflow_step', label: 'Landing Progress'},
                    {key: 'status', label: 'Status'}
                ],
                data = [],
                main_table;

            main_table = new Y.DataTable({
                columnset: columns,
                recordset: data
            });

            main_table.render(div_id);

            Y.webui.main_table = main_table;
        }
    };

    Y.webui.get_workflow_steps();
    Y.webui.get_workflow_statuses();
}, '0.0.1', {requires: ['io', 'node', 'json-parse', 'datatable']});
